@@ -45,10 +45,10 @@
 /* Loading.  */
 
 static int
-impl_load (BonoboPersistFile *pf,
-	   const CORBA_char *filename,
-	   CORBA_Environment *ev,
-	   void *closure)
+pf_impl_load (BonoboPersistFile *pf,
+	      const CORBA_char *filename,
+	      CORBA_Environment *ev,
+	      void *closure)
 {
 	GtkHTML *html;
 	GtkHTMLStream *stream;
@@ -67,7 +67,7 @@ impl_load (BonoboPersistFile *pf,
 	if (was_editable)
 		gtk_html_set_editable (html, FALSE);
 
-	stream = gtk_html_begin (html);
+	stream = gtk_html_begin_content (html, "text/html; charset=utf-8");
 	if (stream == NULL) {
 		close (fd);
 		if (was_editable)
@@ -119,13 +119,14 @@ save_receiver  (const HTMLEngine *engine,
 			return FALSE;
 
 		len -= count;
+		data += count;
 	}
 
 	return TRUE;
 }
 
 static int
-impl_save (BonoboPersistFile *pf,
+pf_impl_save (BonoboPersistFile *pf,
 	   const CORBA_char *filename,
 	   CORBA_Environment *ev,
 	   void *closure)
@@ -136,12 +137,12 @@ impl_save (BonoboPersistFile *pf,
 
 	html = GTK_HTML (closure);
 
-	fd = open (filename, O_WRONLY | O_CREAT, 0600);
+	fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0600);
 
 	if (fd == -1)
 		return -1;
 
-	if (! gtk_html_save (html, save_receiver, GINT_TO_POINTER (fd)))
+	if (!gtk_html_save (html, (GtkHTMLSaveReceiverFn)save_receiver, GINT_TO_POINTER (fd)))
 		retval = -1;
 	else
 		retval = 0;
@@ -151,9 +152,30 @@ impl_save (BonoboPersistFile *pf,
 	return retval;
 }
 
+static void
+pf_destroy (BonoboPersistFile *pf, GtkHTML *html)
+{
+	g_return_if_fail (GTK_IS_HTML (html));
+	
+	gtk_object_unref (GTK_OBJECT (html));
+}
+
 
 BonoboPersistFile *
 persist_file_impl_new (GtkHTML *html)
 {
-	return bonobo_persist_file_new (impl_load, impl_save, html);
+	BonoboPersistFile *pf;
+	
+	gtk_object_ref (GTK_OBJECT (html));
+
+	pf = bonobo_persist_file_new (pf_impl_load, pf_impl_save, html);
+
+	gtk_signal_connect (GTK_OBJECT (pf), "destroy",
+			    pf_destroy, html);
+
+	return pf;
 }
+
+
+
+
