@@ -149,7 +149,9 @@ select_object (HTMLObject *o, HTMLEngine *e, gpointer data)
 	if (o == i->from.object)
 		etop->selected_in = TRUE;
 	if (etop->selected_in)
-		html_object_select_range (o, e, html_interval_get_start (i, o), html_interval_get_length (i, o), TRUE);
+		html_object_select_range (o, e,
+					  html_interval_get_start (i, o), html_interval_get_length (i, o),
+					  !html_engine_frozen (e));
 
 	if (o == i->to.object)
 		etop->selected_in = FALSE;
@@ -167,7 +169,7 @@ html_interval_select (HTMLInterval *i, HTMLEngine *e)
 static void
 unselect_object (HTMLObject *o, HTMLEngine *e, gpointer data)
 {
-	html_object_select_range (o, e, 0, 0, TRUE);
+	html_object_select_range (o, e, 0, 0, !html_engine_frozen (e));
 }
 
 void
@@ -331,8 +333,15 @@ html_point_max (HTMLPoint *a, HTMLPoint *b)
 	b_downline = get_downtree_line (b->object);
 	do_downtree_lines_intersection (&a_downline, &b_downline, NULL);
 
-	rv = html_object_children_max (HTML_OBJECT (a_downline->data), HTML_OBJECT (b_downline->data))
-		== HTML_OBJECT (a_downline->data) ? a : b;
+	if (a_downline == NULL)
+		/* it means that a is parent (container) of b */
+		rv = a->offset ? a : b;
+	else if (b_downline == NULL)
+		/* it means that b is parent (container) of a */
+		rv = b->offset ? b : a;
+	else
+		rv = html_object_children_max (HTML_OBJECT (a_downline->data), HTML_OBJECT (b_downline->data))
+			== HTML_OBJECT (a_downline->data) ? a : b;
 	g_slist_free (a_downline);
 	g_slist_free (b_downline);
 
