@@ -73,6 +73,12 @@ gchar *h_utf8_offset_to_pointer  (const gchar *str, gint         offset);
 #define g_utf8_pointer_to_offset h_utf8_pointer_to_offset
 #endif
 
+static inline gboolean
+is_in_the_save_cluev (HTMLObject *text, HTMLObject *o)
+{
+	return html_object_nth_parent (o, 2) == html_object_nth_parent (text, 2);
+}
+
 static void
 get_tags (const HTMLText *text,
 	  const HTMLEngineSaveState *state,
@@ -84,7 +90,7 @@ get_tags (const HTMLText *text,
 	HTMLObject *prev, *next;
 	HTMLText *pt = NULL, *nt = NULL;
 	gboolean font_tag = FALSE;
-	gboolean std_color, std_size, n_std_color, n_std_size;
+	gboolean std_color, std_size;
 
 	font_style = text->font_style;
 
@@ -99,9 +105,9 @@ get_tags (const HTMLText *text,
 	while (next && !html_object_is_text (next))
 		next = html_object_next_cursor_leaf (next, state->engine);
 
-	if (prev && html_object_is_text (prev))
+	if (prev && is_in_the_save_cluev (HTML_OBJECT (text), prev) && html_object_is_text (prev))
 		pt = HTML_TEXT (prev);
-	if (next && html_object_is_text (next))
+	if (next && is_in_the_save_cluev (HTML_OBJECT (text), next) && html_object_is_text (next))
 		nt = HTML_TEXT (next);
 
 	/* font tag */
@@ -109,17 +115,13 @@ get_tags (const HTMLText *text,
 									     HTMLTextColor))
 		|| html_color_equal (text->color,
 				      html_colorset_get_color (state->engine->settings->color_set, HTMLLinkColor));
-	n_std_color = nt && (html_color_equal (nt->color, html_colorset_get_color (state->engine->settings->color_set,
-										  HTMLTextColor))
-			     || html_color_equal (nt->color,
-						  html_colorset_get_color (state->engine->settings->color_set,
-									   HTMLLinkColor)));
 	std_size = (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK) == 0;
-	n_std_size = nt && (nt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK) == 0;
 
-	if ((!std_color && (!pt || !html_color_equal (text->color, pt->color)))
-	    || (!std_size && (!pt || (pt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK)
-			      != (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK)))) {
+
+	if ((!std_color || !std_size)
+	    && (!pt
+		|| !html_color_equal (text->color, pt->color)
+		|| (pt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK) != (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK))) {
 		if (!std_color) {
 			g_string_sprintfa (ot, "<FONT COLOR=\"#%02x%02x%02x\"",
 					   text->color->color.red   >> 8,
@@ -135,11 +137,10 @@ get_tags (const HTMLText *text,
 		g_string_append_c (ot, '>');
 	}
 
-	if (((!std_color && (!nt || !html_color_equal (text->color, nt->color)))
-	     || (std_color && nt && !n_std_color))
-	    || ((!std_size && (!nt || (nt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK)
-			       != (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK)))
-		|| (std_size && nt && !n_std_size))) {
+	if ((!std_color || !std_size)
+	    && (!nt
+		|| !html_color_equal (text->color, nt->color)
+		|| (nt->font_style & GTK_HTML_FONT_STYLE_SIZE_MASK) != (font_style & GTK_HTML_FONT_STYLE_SIZE_MASK))) {
 		g_string_append (ct, "</FONT>");
 	}
 
@@ -1233,7 +1234,7 @@ get_cursor (HTMLObject *self,
 	guint ascent, descent;
 
 	next = html_object_next_not_slave (self);
-	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next)) {
+	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next) && HTML_TEXT (next)->text [0] != ' ') {
 		html_object_get_cursor (next, painter, 0, x1, y1, x2, y2);
 		return;
 	}
@@ -1263,7 +1264,7 @@ get_cursor_base (HTMLObject *self,
 	HTMLObject *obj, *next;
 
 	next = html_object_next_not_slave (self);
-	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next)) {
+	if (offset == HTML_TEXT (self)->text_len && next && html_object_is_text (next) && HTML_TEXT (next)->text [0] != ' ') {
 		html_object_get_cursor_base (next, painter, 0, x, y);
 		return;
 	}
@@ -1280,7 +1281,7 @@ get_cursor_base (HTMLObject *self,
 		    || obj->next == NULL
 		    || HTML_OBJECT_TYPE (obj->next) != HTML_TYPE_TEXTSLAVE) {
 			html_object_calc_abs_position (obj, x, y);
-			if (offset != slave->posStart) {
+			if (offset > slave->posStart) {
 				HTMLText *text;
 				GtkHTMLFontStyle font_style;
 				gint line_offset;
